@@ -1,7 +1,7 @@
 # -*- coding: latin-1 -*-
-# Copyright (c) 2008-2009 Michael Howitz
+# Copyright (c) 2008-2011 Michael Howitz
 # See also LICENSE.txt
-# $Id: adapter.py 459 2009-05-12 18:39:39Z mac $
+# $Id: adapter.py 1229 2011-01-29 17:07:16Z icemac $
 
 import zope.interface
 import zope.schema.interfaces
@@ -26,6 +26,12 @@ def title_for_choice_value(field, value):
     return factory.getTitle(value)
 
 
+@zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
+def obj_dot_title(obj):
+    "ITitle adapter for objects those title is stored on the title attribute."
+    return obj.title
+
+
 @zope.component.adapter(zope.interface.Interface)
 @zope.interface.implementer(icemac.addressbook.interfaces.ITitle)
 def default_title(obj):
