@@ -1,9 +1,9 @@
-# -*- coding: latin-1 -*-
 # Copyright (c) 2008-2011 Michael Howitz
 # See also LICENSE.txt
-# $Id: address.py 1293 2011-08-10 18:41:45Z icemac $
+# $Id: address.py 1427 2011-12-06 07:15:23Z icemac $
 
-from icemac.addressbook.i18n import MessageFactory as _
+from icemac.addressbook.i18n import _
+import grokcore.component
 import icemac.addressbook.entities
 import icemac.addressbook.interfaces
 import persistent
@@ -84,6 +84,14 @@ def email_address_title(email):
     return title
 
 
+@grokcore.component.adapter(icemac.addressbook.interfaces.IPerson)
+@grokcore.component.implementer(icemac.addressbook.interfaces.IEMailAddress)
+def email_address_of_person(person):
+    email_entity = icemac.addressbook.interfaces.IEntity(
+        icemac.addressbook.interfaces.IEMailAddress)
+    return getattr(person, email_entity.tagged_values['default_attrib'])
+
+
 class HomePageAddress(
     persistent.Persistent, zope.container.contained.Contained):
     """A home page address."""
