@@ -20,9 +20,10 @@ import zope.interface
 import zope.security
 import zope.security.interfaces
 import zope.security.proxy
+import zope.session.interfaces
 import zope.site.hooks
 import zope.traversing.api
-import zope.traversing.browser.interfaces
+import zope.traversing.browser
 import zope.traversing.publicationtraverse
 
 
@@ -78,10 +79,7 @@ class BaseView(FlashView):
         super(BaseView, self).update()
 
     def url(self, obj):
-        adapter = zope.component.getMultiAdapter(
-            (obj, self.request),
-            zope.traversing.browser.interfaces.IAbsoluteURL)
-        return adapter()
+        return zope.traversing.browser.absoluteURL(obj, self.request)
 
 
 class BaseForm(BaseView):
@@ -167,8 +165,10 @@ class BaseAddForm(BaseForm, z3c.formui.form.AddForm):
 
 
 def update_with_redirect(class_, self):
-    "Call update of super class and redirect when necessary."
+    """Call update of super class and redirect when necessary."""
     get_needed_resources(self)
+    # Caution: we need the class_ parameter as we get infinite recursion if
+    # using self.__class__ in the super call.
     super(class_, self).update()
     if self.request.response.getStatus() in (302, 303, 304):
         # already redirecting
@@ -346,3 +346,9 @@ def can_access(uri_part):
                 return False
         return True
     return can_access_form
+
+
+def get_session(request):
+    """Get the browser session of the current user."""
+    return zope.session.interfaces.ISession(request)[
+        icemac.addressbook.interfaces.PACKAGE_ID]
