@@ -1,7 +1,6 @@
-# -*- coding: latin-1 -*-
-# Copyright (c) 2008-2010 Michael Howitz
+# Copyright (c) 2008-2011 Michael Howitz
 # See also LICENSE.txt
-# $Id: interfaces.py 1164 2010-11-17 16:54:23Z icemac $
+# $Id: interfaces.py 1328 2011-09-22 19:42:28Z icemac $
 
 from icemac.addressbook.i18n import _
 import z3c.form.interfaces
@@ -29,3 +28,10 @@ class IPersonCount(zope.interface.Interface):
 
     count = zope.schema.Int(title=_(u'number of persons'), required=False)
     notes = zope.schema.TextLine(title=_(u'notes'), required=False)
+
+
+class IErrorMessage(zope.interface.Interface):
+    """Render error message human readable."""
+
+    def __unicode__():
+        """Returns the translateable error text."""
