@@ -1,17 +1,19 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2009-2010 Michael Howitz
+# Copyright (c) 2009-2011 Michael Howitz
 # See also LICENSE.txt
-# $Id: principals.py 799 2010-01-23 14:24:17Z icemac $
+# $Id: principals.py 1294 2011-08-10 18:42:54Z icemac $
 
 from icemac.addressbook.i18n import MessageFactory as _
 import icemac.addressbook.browser.base
 import icemac.addressbook.browser.metadata
 import icemac.addressbook.browser.table
+import icemac.addressbook.interfaces
 import icemac.addressbook.principals.interfaces
 import icemac.addressbook.principals.principals
 import transaction
 import z3c.form.button
 import z3c.form.field
+import z3c.form.validator
 import z3c.form.widget
 import zope.interface
 import zope.schema
@@ -59,16 +61,11 @@ class PersonFieldDataManager(z3c.form.datamanager.AttributeField):
         self.context.person = value
 
 
-def get_principal_entity():
-    entities_util = icemac.addressbook.browser.base.get_entities_util()
-    return entities_util.getEntity(
-        icemac.addressbook.principals.interfaces.IPrincipal)
-
-
 class AddForm(icemac.addressbook.browser.base.BaseAddForm):
 
     label = _(u'Add new user')
     class_ = icemac.addressbook.principals.principals.Principal
+    interface = icemac.addressbook.principals.interfaces.IPrincipal
     next_url = 'parent'
 
     @property
@@ -85,7 +82,8 @@ class AddForm(icemac.addressbook.browser.base.BaseAddForm):
                 icemac.addressbook.principals.interfaces.IRoles)
             +
             z3c.form.field.Fields(
-                *get_principal_entity().getFieldValuesInOrder()).omit(
+                *icemac.addressbook.interfaces.IEntity(
+                    self.interface).getFieldValues()).omit(
                 'login', 'person')
             )
 
@@ -110,7 +108,9 @@ password_not_required = z3c.form.widget.StaticWidgetAttribute(
 
 class EditForm(icemac.addressbook.browser.base.GroupEditForm):
 
+    label = _(u'Edit user')
     groups = (icemac.addressbook.browser.metadata.MetadataGroup,)
+    interface = icemac.addressbook.principals.interfaces.IPrincipal
     next_url = 'parent'
 
     @property
@@ -127,7 +127,8 @@ class EditForm(icemac.addressbook.browser.base.GroupEditForm):
                 icemac.addressbook.principals.interfaces.IRoles)
             +
             z3c.form.field.Fields(
-                *get_principal_entity().getFieldValuesInOrder()).omit(
+                *icemac.addressbook.interfaces.IEntity(
+                    self.interface).getFieldValues()).omit(
                 'person', 'login')
             )
 
@@ -149,13 +150,45 @@ class EditForm(icemac.addressbook.browser.base.GroupEditForm):
         self.redirect_to_next_url('object', '@@delete_user.html')
 
     def applyChanges(self, data):
+        current_principal_id = (
+            self.request.interaction.participations[0].principal.id)
+        current_login_name = None
+        auth = zope.component.getUtility(
+            zope.authentication.interfaces.IAuthentication)
+        for key, plugin in auth.getAuthenticatorPlugins():
+            principal_info = plugin.principalInfo(current_principal_id)
+            if principal_info is not None:
+                current_login_name = principal_info.login
+                break
+
+        old_login_name = self.context.login
+        editing_own_data = (old_login_name == current_login_name)
         try:
-            super(EditForm, self).applyChanges(data)
+            changes = super(EditForm, self).applyChanges(data)
         except ValueError, e:
             transaction.doom()
             raise z3c.form.interfaces.ActionExecutionError(
                 zope.interface.Invalid(_(e.args[0])))
 
+        if not editing_own_data:
+            # User is not editing his own data principal's data:
+            return changes
+
+        changed_field_names = []
+        changed_field_names.extend(
+            changes.get(
+                icemac.addressbook.principals.interfaces.IPrincipal, []))
+        changed_field_names.extend(
+            changes.get(
+                icemac.addressbook.principals.interfaces.IPasswordFields, []))
+        if 'login' in changed_field_names:
+            self.send_flash(_('You changed the login name, please re-login.'))
+        if ('password' in changed_field_names and
+            'password_repetition' in changed_field_names and
+            data['password']):
+            self.send_flash(_('You changed the password, please re-login.'))
+        return changes
+
 
 class EditForm_password_Validator(z3c.form.validator.SimpleFieldValidator):
     """Validator for data field in add form."""
