@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2009-2010 Michael Howitz
+# Copyright (c) 2009-2011 Michael Howitz
 # See also LICENSE.txt
+import grokcore.component
 import icemac.addressbook.interfaces
 import persistent
 import persistent.interfaces
@@ -9,6 +10,8 @@ import zope.container.contained
 import zope.dottedname.resolve
 import zope.interface.interfaces
 import zope.schema
+import zope.schema.interfaces
+import zope.security.proxy
 
 
 MAIN_ENTITIES_NAME_SUFFIXES = [
@@ -161,6 +164,8 @@ class ChoiceFieldValuesSource(zc.sourcefactory.basic.BasicSourceFactory):
         return value
 
 
+@grokcore.component.adapter(icemac.addressbook.interfaces.IField)
+@grokcore.component.implementer(zope.schema.interfaces.IField)
 def user_field_to_schema_field(field):
     """Convert a user defined field (IField) into a zope.schema field."""
     if field.type == 'Choice':
@@ -176,6 +181,28 @@ def user_field_to_schema_field(field):
     return schema_field
 
 
+def index(key, list, default):
+    "Index of `key` in `list` but `default` when it is not in `list`."
+    try:
+        return list.index(key)
+    except ValueError:
+        return default
+
+
+def sorted_fields(fields, field_order):
+    """Returns the fields sorted by their index in the `field_order`.
+
+    Fields which are not in the `field_order` are sorted to the end
+    accordingly to their position in the `fields` list.
+
+    """
+    raw_field_order = [x[0] for x in fields]
+    return sorted(
+        fields,
+        key=lambda (name, field): index(
+            name, field_order, 100000 + raw_field_order.index(name)))
+
+
 class FakeObject(object):
     "We need an instance to provide an interface for the `getAdapters` call."
 
@@ -183,9 +210,13 @@ class FakeObject(object):
 class Entity(object):
     "An entity int the address book universe."
     # Use `create_entity` factory for easier usage.
+    # Caution: This class is only a base class, in most cases you will use
+    #          EditableEntity (see below).
 
     zope.interface.implements(icemac.addressbook.interfaces.IEntity)
 
+    # IEntityRead
+
     title = zope.schema.fieldproperty.FieldProperty(
         icemac.addressbook.interfaces.IEntity['title'])
     interface = zope.schema.fieldproperty.FieldProperty(
@@ -199,8 +230,9 @@ class Entity(object):
         self.title = title
         self.interface = interface
         self.class_name = class_name
-        # getRawFields needs an instance which provides the interface to
-        # look up the user defined fields which are named adapters
+        # _get_raw_fields_unordered needs an instance which provides the
+        # interface to look up the user defined fields which are named
+        # adapters:
         self._fake_object = FakeObject()
         zope.interface.directlyProvides(self._fake_object, self.interface)
         # Additional keyword arguments are stored as tagged values
@@ -221,48 +253,47 @@ class Entity(object):
         "Dict of tagged values of the entity."
         return self._tagged_values.copy()
 
-    def getRawFields(self):
-        """Get ordered name, field tuples of the schema fields on the entity.
+    def getRawFields(self, sorted=True):
+        """Get (name, field) tuples of the schema fields on the entity.
 
-        Returnes static (zope.schema) and user defined (IField) fields.
+        When `sorted` is true the fields are sorted using the field order.
+
+        Returns static (zope.schema) and user defined (IField) fields.
 
         """
-        for name, field in zope.schema.getFieldsInOrder(self.interface):
-            yield name, field
-        # self._fake_object is needed here as the interfaces provided by the
-        # objects are used in the look up
-        adapters = zope.component.getAdapters(
-            (self, self._fake_object), icemac.addressbook.interfaces.IField)
-        for name, field in adapters:
-            yield str(field.__name__), field
+        raw_fields = self._get_raw_fields_unordered()
+        if not sorted:
+            return raw_fields
+        return sorted_fields(list(raw_fields), self.getFieldOrder())
 
-    def getFieldsInOrder(self):
-        """Get ordered name, field tuples of the schema fields on the entity.
+    def getFields(self, sorted=True):
+        """Get (name, field) tuples of the schema fields on the entity.
+
+        When `sorted` is true the fields are sorted using the field order.
 
         Converts user defined fields (see IField) into zope.schema fields.
 
         """
-        for name, field in self.getRawFields():
-            if icemac.addressbook.interfaces.IField.providedBy(field):
-                yield name, user_field_to_schema_field(field)
-            else:
-                yield name, field
+        for name, field in self.getRawFields(sorted=sorted):
+            yield name, zope.schema.interfaces.IField(field)
 
-    def getFieldValuesInOrder(self):
-        """Get ordered list of the schema fields on the entity.
+    def getFieldValues(self, sorted=True):
+        """Get list of the schema fields on the entity.
+
+        When `sorted` is true the fields are sorted using the field order.
 
         Converts user defined fields (see IField) into zope.schema fields.
 
         """
-        return [field for name, field in self.getFieldsInOrder()]
+        return [field for name, field in self.getFields(sorted=sorted)]
 
     def getRawField(self, field_name):
         """Get a field by its name."""
-        return dict(self.getRawFields())[field_name]
+        return dict(self.getRawFields(sorted=False))[field_name]
 
     def getField(self, field_name):
         """Get a zope.schema field by its name."""
-        return dict(self.getFieldsInOrder())[field_name]
+        return dict(self.getFields(sorted=False))[field_name]
 
     def getClass(self):
         """Get the class object for `self.class_name`."""
@@ -270,11 +301,83 @@ class Entity(object):
             return zope.dottedname.resolve.resolve(self.class_name)
         raise ValueError("class_name is not set.")
 
+    def getFieldOrder(self):
+        """Get the ordered names of the fields."""
+        order_storage = zope.component.queryUtility(
+            icemac.addressbook.interfaces.IOrderStorage)
+        try:
+            return order_storage.__iter__(self._order_storage_namespace)
+        except (KeyError, ValueError, AttributeError):
+            # Either the order_storage is None or the namespace cannot be
+            # computed or it is unknown, so we can't order the fields.
+            return []
+
+    # IEntityWrite
+
+    def addField(self, field):
+        """Add a user defined field to the entity."""
+        # The field needs to be stored in the entities utility:
+        parent = zope.component.getUtility(
+            icemac.addressbook.interfaces.IEntities)
+        name = icemac.addressbook.utils.add(parent, field)
+        # The field needs to be an adapter:
+        sm = zope.site.hooks.getSiteManager()
+        sm.registerAdapter(
+            FieldAdapterFactory(field),
+            provided=icemac.addressbook.interfaces.IField,
+            required=(icemac.addressbook.interfaces.IEntity, self.interface),
+            name=name)
+        return name
+
+    def setFieldOrder(self, field_names):
+        """Update the order of the fields like in `field_names`."""
+        order_storage = zope.component.getUtility(
+            icemac.addressbook.interfaces.IOrderStorage)
+        existing_field_names = [x[0] for x in self._get_raw_fields_unordered()]
+        ns = self._order_storage_namespace
+        # Removing exising contents of namespace in order storage while
+        # creating the namespace when it does not yet exist
+        order_storage.truncate(ns)
+        for name in field_names:
+            if name not in existing_field_names:
+                # We do not allow to write arbitrary data here:
+                continue
+            order_storage.add(name, ns)
+
+    # private
+
+    @property
+    def _order_storage_namespace(self):
+        "Get the name space used in the order storage."
+        return '%s%s' % (
+            icemac.addressbook.interfaces.FIELD_NS_PREFIX, self.name)
+
+    def _get_raw_fields_unordered(self):
+        "Get the raw fields not ordered."
+        for name, field in zope.schema.getFieldsInOrder(self.interface):
+            yield name, field
+        # self._fake_object is needed here as the interfaces provided by the
+        # objects are used in the look-up
+        adapters = zope.component.getAdapters(
+            (self, self._fake_object), icemac.addressbook.interfaces.IField)
+        for name, field in adapters:
+            yield str(field.__name__), field
+
+
+class EditableEntity(Entity):
+    """Special entity which is editable.
+
+    This means that new fields can be added and the fields can be sorted.
+
+    """
+    zope.interface.implementsOnly(
+        icemac.addressbook.interfaces.IEditableEntity)
+
 
 def create_entity(title, interface, class_, **kw):
-    "Factory to create an entity and to the ZCA set up."
+    "Factory to create an editable entity and to the ZCA set up."
     class_name = '%s.%s' % (class_.__module__, class_.__name__)
-    entity = Entity(title, interface, class_name, **kw)
+    entity = EditableEntity(title, interface, class_name, **kw)
     zope.interface.classImplements(
         class_, icemac.addressbook.interfaces.IMayHaveUserFields)
     return entity
@@ -305,21 +408,6 @@ class FieldAdapterFactory(persistent.Persistent):
         return self._field
 
 
-def store_and_register_field(field, entity):
-    "Store a `field` in the ZODB and register it for an `entity`."
-    # store in entities utility
-    parent = zope.component.getUtility(icemac.addressbook.interfaces.IEntities)
-    name = icemac.addressbook.utils.add(parent, field)
-    # register as adapter
-    sm = zope.site.hooks.getSiteManager()
-    sm.registerAdapter(
-        FieldAdapterFactory(field),
-        provided=icemac.addressbook.interfaces.IField,
-        required=(icemac.addressbook.interfaces.IEntity, entity.interface),
-        name=name)
-    return name
-
-
 class FieldStorage(persistent.Persistent):
     """Storage for field values in annotations."""
     zope.component.adapts(
@@ -340,3 +428,20 @@ class FieldStorage(persistent.Persistent):
 
 field_storage = zope.annotation.factory(
     FieldStorage, key='icemac.userfield.storage')
+
+
+def get_bound_schema_field(obj, entity, field):
+    """Returns a bound zope.schema field for `entity` and `field` on `obj`."""
+    if not entity.interface.providedBy(obj):
+        # If the entity is for another object, we expect to find the entity
+        # on a default_attrib.
+        obj = getattr(obj, entity.tagged_values['default_attrib'])
+    if icemac.addressbook.interfaces.IField.providedBy(field):
+        # User defined fields need to be adapted. Additionally security
+        # proxy must be removed as otherwise access fails. This might be a
+        # security hole if access to this function is not properly
+        # protected, but there is no other way.
+        obj = zope.security.proxy.getObject(obj)
+        obj = icemac.addressbook.interfaces.IUserFieldStorage(obj)
+        field = zope.schema.interfaces.IField(field)
+    return field.bind(obj)
