@@ -1,11 +1,11 @@
 # -*- coding: latin-1 -*-
-# Copyright (c) 2008-2010 Michael Howitz
+# Copyright (c) 2008-2011 Michael Howitz
 # See also LICENSE.txt
-# $Id: evolve4.py 783 2010-01-09 14:28:06Z icemac $
+# $Id: evolve4.py 1229 2011-01-29 17:07:16Z icemac $
 
 __docformat__ = "reStructuredText"
 
-import zope.app.generations.utility
+import zope.generations.utility
 
 
 generation = 4
@@ -15,6 +15,6 @@ def evolve(context):
     """Update the root folder to be a ``zope.site.folder.Folder`` instead of
        ``zope.app.folder.folder.Folder``.
     """
-    root = zope.app.generations.utility.getRootFolder(context)
+    root = zope.generations.utility.getRootFolder(context)
     context.connection.root()._p_changed = True
     root._p_changed = True
