@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2008-2009 Michael Howitz
+# Copyright (c) 2008-2011 Michael Howitz
 # See also LICENSE.txt
-# $Id: person.py 688 2009-11-21 15:07:47Z icemac $
+# $Id: person.py 1257 2011-03-05 14:18:16Z icemac $
 
 from icemac.addressbook.i18n import MessageFactory as _
 import gocept.reference
+import icemac.addressbook.address
 import icemac.addressbook.entities
 import icemac.addressbook.interfaces
 import zope.annotation.interfaces
@@ -33,7 +34,7 @@ class Person(zope.container.btree.BTreeContainer):
     keywords = gocept.reference.ReferenceCollection(
         'keywords', ensure_integrity=True)
     default_postal_address = gocept.reference.Reference(
-        'default_postal_address',ensure_integrity=True)
+        'default_postal_address', ensure_integrity=True)
     default_email_address = gocept.reference.Reference(
         'default_email_address', ensure_integrity=True)
     default_home_page_address = gocept.reference.Reference(
@@ -59,6 +60,7 @@ def title(person):
 
 def get_default_field(interface):
     "Find the field of a default value attribute on person."
+    # XXX use IEntity(interface).taggedValues['default_attrib'] here:
     names_descrs = (
         icemac.addressbook.interfaces.IPersonDefaults.namesAndDescriptions())
     for name, descr in names_descrs:
@@ -66,6 +68,33 @@ def get_default_field(interface):
             return icemac.addressbook.interfaces.IPersonDefaults[name]
 
 
+def sorted_person_defaults(name_field_tuples):
+    """Sorts the `name_field_tuples` as defined in entity order."""
+    entity_order = zope.component.getUtility(
+        icemac.addressbook.interfaces.IEntityOrder)
+    return sorted(
+        name_field_tuples,
+        key=lambda (name, field): entity_order.get(
+            icemac.addressbook.address.default_attrib_name_to_entity(name)))
+
+
+class PersonDefaultsEntity(icemac.addressbook.entities.Entity):
+    "Entity which sorts the fields in IPersonDefaults alike entity order."
+
+    def getRawFields(self, sorted=True):
+        name_fields = super(
+            PersonDefaultsEntity, self).getRawFields(sorted=False)
+        if sorted:
+            return sorted_person_defaults(name_fields)
+        return name_fields
+
+
+person_defaults_entity = PersonDefaultsEntity(
+    _(u'main adresses and numbers'),
+    icemac.addressbook.interfaces.IPersonDefaults,
+    'icemac.addressbook.person.PersonDefaults')
+
+
 @zope.component.adapter(icemac.addressbook.interfaces.IPerson,
                         zope.lifecycleevent.IObjectCreatedEvent)
 def person_created(person, event):
@@ -93,3 +122,17 @@ class Keywords(object):
     def get_titles(self):
         return [icemac.addressbook.interfaces.ITitle(x)
                 for x in self.context.keywords]
+
+
+class PersonName(object):
+
+    zope.interface.implements(icemac.addressbook.interfaces.IPersonName)
+    zope.component.adapts(icemac.addressbook.interfaces.IPerson)
+
+    def __init__(self, person):
+        self.person = person
+
+    def get_name(self):
+        values = [self.person.first_name, self.person.last_name]
+        result = [x for x in values if x]
+        return ' '.join(result)
