@@ -3,26 +3,27 @@
 # See also LICENSE.txt
 
 from icemac.addressbook.i18n import _
-import icemac.addressbook.preferences.sources
+import icemac.addressbook.fieldsource
+import icemac.addressbook.sources
 import zope.interface
 import zope.schema
 
 
 class IPersonList(zope.interface.Interface):
-    "Person list preferences."
+    """Person list preferences."""
 
     columns = zope.schema.List(
         title=_('columns'), required=True,
         value_type=zope.schema.Choice(
             title=_('columns'),
-            source=icemac.addressbook.preferences.sources.column_source))
+            source=icemac.addressbook.fieldsource.source))
 
     order_by = zope.schema.Choice(
         title=_('order by'),
-        source=icemac.addressbook.preferences.sources.column_source)
+        source=icemac.addressbook.fieldsource.source)
 
     sort_direction = zope.schema.Choice(
         title=_('sort direction'),
-        source=icemac.addressbook.preferences.sources.asc_des_csource)
+        source=icemac.addressbook.sources.asc_desc_source)
 
     batch_size = zope.schema.Int(title=_('batch size'), min=1)
