@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2009-2011 Michael Howitz
 # See also LICENSE.txt
-# $Id: testing.py 1370 2011-11-03 19:36:53Z icemac $
+# $Id: testing.py 1425 2011-12-05 21:08:55Z icemac $
 
-import gocept.selenium.grok
+import gocept.selenium.wsgi
 import icemac.addressbook.address
 import icemac.addressbook.addressbook
 import icemac.addressbook.file.file
@@ -15,6 +15,7 @@ import icemac.addressbook.principals.sources
 import icemac.addressbook.startup
 import icemac.addressbook.utils
 import inspect
+import lxml.etree
 import os
 import os.path
 import plone.testing
@@ -64,31 +65,27 @@ class _AddressBookUnitTests(plone.testing.Layer):
 ADDRESS_BOOK_UNITTESTS = _AddressBookUnitTests(name='AddressBookUnitTests')
 
 
-class _ZCMLAndZODBLayer(plone.testing.zodb.EmptyZODB):
-    """Layer which sets up ZCML and ZODB to be useable for Zope 3."""
-    defaultBases = (plone.testing.zca.LAYER_CLEANUP,)
+ZCML_LAYER = plone.testing.zca.ZCMLSandbox(
+    name="AddressBookZCML", filename="ftesting.zcml", module=__name__,
+    package=icemac.addressbook)
 
-    package = icemac.addressbook
+
+class _ZODBLayer(plone.testing.zodb.EmptyZODB):
+    """Layer which sets up ZODB to be useable for Zope 3."""
+    defaultBases = (ZCML_LAYER,)
 
     def setUp(self):
-        super(_ZCMLAndZODBLayer, self).setUp()
-        plone.testing.zca.setUpZcmlFiles(
-            [("ftesting.zcml", self.package)])
+        super(_ZODBLayer, self).setUp()
         zope.event.notify(zope.processlifetime.DatabaseOpened(self['zodbDB']))
         transaction.commit()
 
-    def tearDown(self):
-        plone.testing.zca.tearDownZcmlFiles()
-        super(_ZCMLAndZODBLayer, self).tearDown()
-
     def testSetUp(self):
         pass
 
     def testTearDown(self):
         pass
 
-
-ZCML_AND_ZODB_LAYER = _ZCMLAndZODBLayer(name='ZCMLAndZODBLayer')
+ZODB_LAYER = _ZODBLayer(name='ZODBLayer')
 
 
 def setUpStackedDemoStorage(self, name):
@@ -125,7 +122,7 @@ def tearDownStackedDemoStorage(self):
 
 class _ZODBTestLayer(plone.testing.Layer):
     """Layer which opens the ZODB for each test."""
-    defaultBases = (ZCML_AND_ZODB_LAYER,)
+    defaultBases = (ZODB_LAYER,)
 
     def testSetUp(self):
         setUpZODBConnection(self)
@@ -138,7 +135,7 @@ FUNCTIONAL_LAYER = _ZODBTestLayer(name='ZODBTestLayer')
 
 class _ZODBIsolatedTestLayer(plone.testing.Layer):
     """Layer which puts a DemoStorage on ZODB for each test."""
-    defaultBases = (ZCML_AND_ZODB_LAYER,)
+    defaultBases = (ZODB_LAYER,)
 
     def testSetUp(self):
         setUpStackedDemoStorage(self, self.__name__)
@@ -165,6 +162,14 @@ class WSGILayer(plone.testing.Layer):
     def tearDown(self):
         del self['wsgi_app']
 
+    def testSetUp(self):
+        # The layer has to store the database at layer set up (depending
+        # layers require this) but we get a new database at test set up, so
+        # we have to set the right ZODB here:
+        self['wsgi_app'].requestFactory = (
+            zope.app.publication.httpfactory.HTTPPublicationRequestFactory(
+                self['zodbDB']))
+
 WSGI_LAYER = WSGILayer()
 
 
@@ -183,19 +188,25 @@ class _WSGITestBrowserLayer(zope.testbrowser.wsgi.Layer,
             zope.app.wsgi.testlayer.TransactionMiddleware(
                 getRootFolder, self['wsgi_app']))
 
-    def testSetUp(self):
-        # WSGI app stores the database at layer set up but we get a new
-        # database at test set up, so we have to set the right ZODB here:
-        self['wsgi_app'].requestFactory = (
-            zope.app.publication.httpfactory.HTTPPublicationRequestFactory(
-                self['zodbDB']))
-
-
 WSGI_TEST_BROWSER_LAYER = _WSGITestBrowserLayer(name='WSGITestBrowserLayer')
 
 
-SeleniumLayer = gocept.selenium.grok.Layer(
-    icemac.addressbook, name='SeleniumLayer')
+class GoceptSeleniumPloneTestingIntegrationLayer(gocept.selenium.wsgi.Layer,
+                                                 plone.testing.Layer):
+    """Layer which integrates gocept.selenium with plone.testing."""
+
+    defaultBases = (WSGI_LAYER, )
+
+    def __init__(self, *args, **kw):
+        # WSGI application is set up in base layers so we cannot access it
+        # here yet:
+        gocept.selenium.wsgi.Layer.__init__(self, application=None)
+        plone.testing.Layer.__init__(self, *args, **kw)
+
+    def setup_wsgi_stack(self, app):
+        return self['wsgi_app']
+
+SELENIUM_LAYER = GoceptSeleniumPloneTestingIntegrationLayer()
 
 
 def setUpAddressBook(self):
@@ -210,7 +221,7 @@ def setUpAddressBook(self):
 
 class _AddressBookFunctionalLayer(plone.testing.Layer):
     "Layer where the address book gets created in the layer set up."
-    defaultBases = (ZCML_AND_ZODB_LAYER,)
+    defaultBases = (ZODB_LAYER,)
 
     def setUp(self):
         setUpStackedDemoStorage(self, 'AddressBookFunctionalTestCase')
@@ -352,6 +363,12 @@ class Browser(z3c.etestbrowser.wsgi.ExtendedTestBrowser):
         self.addHeader('Authorization', 'Basic %s:%s' %
                        (username, USERNAME_PASSWORD_MAP[username]))
 
+    def etree_to_list(self, etree):
+        """"Convert an etree into a list (lines without leading whitespace.)"""
+        return [x.strip()
+                for x in lxml.etree.tostring(etree).split('\n')
+                if x.strip()]
+
     get_messages = get_messages
 
 
