@@ -1,5 +1,5 @@
 # -*- coding: latin-1 -*-
-# Copyright (c) 2008-2010 Michael Howitz
+# Copyright (c) 2008-2011 Michael Howitz
 # See also LICENSE.txt
 
 import gocept.reference.verify
@@ -29,6 +29,11 @@ class TestInterfaces(unittest.TestCase):
         gocept.reference.verify.verifyObject(
             icemac.addressbook.interfaces.IPersonDefaults, person)
 
+    def test_person_name(self):
+        zope.interface.verify.verifyObject(
+            icemac.addressbook.interfaces.IPersonName,
+            icemac.addressbook.person.PersonName(None))
+
     def test_address_book(self):
         zope.interface.verify.verifyObject(
             icemac.addressbook.interfaces.IAddressBook,
@@ -85,10 +90,12 @@ class TestInterfaces(unittest.TestCase):
     def test_entity(self):
         class IE(zope.interface.Interface):
             pass
+        entity = icemac.addressbook.entities.Entity(
+                u'E', IE, 'icemac.addressbook.TestInterfaces')
         zope.interface.verify.verifyObject(
-            icemac.addressbook.interfaces.IEntity,
-            icemac.addressbook.entities.Entity(
-                u'E', IE, 'icemac.addressbook.TestInterfaces'))
+            icemac.addressbook.interfaces.IEntityRead, entity)
+        zope.interface.verify.verifyObject(
+            icemac.addressbook.interfaces.IEntityWrite, entity)
 
     def test_field(self):
         zope.interface.verify.verifyObject(
